@@ -247,7 +247,8 @@ static void
 usage (int ecode)
 {
   fprintf (stderr, "dbus-binding-tool [--version] [--help]\n");
-  fprintf (stderr, "dbus-binding-tool ---mode=[pretty|glib-server|glib-client] -prefix=SYMBOL_PREFIX [--ignore-unsupported] [--force] [--output=FILE] [\n");
+  fprintf (stderr, "dbus-binding-tool --mode=[pretty|glib-server|glib-client] [--prefix=SYMBOL_PREFIX] [--ignore-unsupported] [--force] [--output=FILE]\n");
+  fprintf (stderr, "dbus-binding-tool --mode=glib-server --prefix=SYMBOL_PREFIX [--ignore-unsupported] [--force] [--output=FILE]\n");
   exit (ecode);
 }
 
@@ -357,7 +358,7 @@ main (int argc, char **argv)
       ++i;
     }
 
-  if (!has_prefix)
+  if (outputmode == DBUS_BINDING_OUTPUT_GLIB_SERVER && !has_prefix)
     usage (1);
 
   error = NULL;
@@ -413,7 +414,7 @@ main (int argc, char **argv)
                                          &error);
       if (node == NULL)
         {
-	  lose_gerror (_("Unable to load \"%s\""), error);
+	  lose (_("Unable to load \"%s\": %s"), filename, error->message);
         }
       else
 	{
