@@ -73,6 +73,8 @@ struct _DBusGProxyPrivate
   GData *signal_signatures;   /**< D-BUS signatures for each signal */
 
   GHashTable *pending_calls;  /**< Calls made on this proxy which have not yet returned */
+
+  int default_timeout; /**< Default timeout to use, see dbus_g_proxy_set_default_timeout */
 };
 
 static void dbus_g_proxy_init               (DBusGProxy      *proxy);
@@ -656,8 +658,15 @@ unassociate_proxies (gpointer key, gpointer val, gpointer user_data)
 	{
 	  if (!priv->for_owner)
 	    {
-	      g_assert (priv->associated);
-	      g_assert (priv->name_call == NULL);
+	      /* If a service appeared and then vanished very quickly,
+	       * it's conceivable we have an inflight request for
+	       * GetNameOwner here.  Cancel it.
+	       * https://bugs.freedesktop.org/show_bug.cgi?id=18573
+	       */
+	      if (priv->name_call)
+		dbus_g_proxy_cancel_call (manager->bus_proxy, priv->name_call);
+
+	      priv->name_call = NULL;
 
 	      priv->associated = FALSE;
 	      manager->unassociated_proxies = g_slist_prepend (manager->unassociated_proxies, proxy);
@@ -903,6 +912,9 @@ dbus_g_proxy_manager_register (DBusGProxyManager *manager,
                                                     NULL);
       /* FIXME - for now we listen for all NameOwnerChanged; once
        * Anders' detail patch lands we should add individual rules
+       * 
+       * NOTE: if you change this, be sure to change the matching
+       * call to dbus_bus_remove_match in dbus_g_proxy_manager_unregister.
        */
       dbus_bus_add_match (manager->connection,
                           "type='signal',sender='" DBUS_SERVICE_DBUS
@@ -1059,6 +1071,17 @@ dbus_g_proxy_manager_unregister (DBusGProxyManager *manager,
     {
       g_hash_table_destroy (manager->proxy_lists);
       manager->proxy_lists = NULL;
+
+      /*
+       * NOTE: if you change this, be sure to change the matching
+       * call to dbus_bus_add_match in dbus_g_proxy_manager_register.
+       */
+      dbus_bus_remove_match (manager->connection,
+                             "type='signal',sender='" DBUS_SERVICE_DBUS
+			     "',path='" DBUS_PATH_DBUS
+			     "',interface='" DBUS_INTERFACE_DBUS
+			     "',member='NameOwnerChanged'",
+			     NULL);
     }
 
   g_free (tri);
@@ -1233,8 +1256,17 @@ dbus_g_proxy_manager_filter (DBusConnection    *connection,
 						  dbus_message_get_interface (message));
 
 	      owner_list = g_hash_table_lookup (manager->proxy_lists, tri);
-	      if (owner_list != NULL)
-		full_list = g_slist_concat (full_list, g_slist_copy (owner_list->proxies));
+	      if (owner_list != NULL) 
+                {
+	          GSList *elt;
+
+	          /* Ignore duplicates when adding to full_list */
+	          for (elt = owner_list->proxies; elt; elt = g_slist_next (elt)) 
+                    {
+	              if (!g_slist_find (full_list, elt->data))
+	                full_list = g_slist_append (full_list, elt->data);
+	            }
+	        }
 	      g_free (tri);
 	    }
 	}
@@ -1315,6 +1347,7 @@ dbus_g_proxy_init (DBusGProxy *proxy)
 				(GDestroyNotify) dbus_pending_call_unref);
   priv->name_call = 0;
   priv->associated = FALSE;
+  priv->default_timeout = -1;
 }
 
 static GObject *
@@ -1419,13 +1452,14 @@ dbus_g_proxy_class_init (DBusGProxyClass *klass)
                   G_TYPE_NONE, 2, DBUS_TYPE_MESSAGE, G_TYPE_POINTER);
 }
 
-static void
+static gboolean
 cancel_pending_call (gpointer key, gpointer val, gpointer data)
 {
-  DBusGProxyCall *call = key;
-  DBusGProxy *proxy = data;
+  DBusPendingCall *pending = val;
 
-  dbus_g_proxy_cancel_call (proxy, call);
+  dbus_pending_call_cancel (pending);
+
+  return TRUE;
 }
 
 static void
@@ -1440,7 +1474,7 @@ dbus_g_proxy_dispose (GObject *object)
     }
 
   /* Cancel outgoing pending calls */
-  g_hash_table_foreach (priv->pending_calls, cancel_pending_call, proxy);
+  g_hash_table_foreach_remove (priv->pending_calls, cancel_pending_call, NULL);
   g_hash_table_destroy (priv->pending_calls);
   priv->pending_calls = NULL;
 
@@ -1620,6 +1654,7 @@ marshal_dbus_message_to_g_marshaller (GClosure     *closure,
   
   {
     DBusGValueMarshalCtx context;
+    context.recursion_depth = 0;
     context.gconnection = DBUS_G_CONNECTION_FROM_CONNECTION (priv->manager->connection);
     context.proxy = proxy;
 
@@ -2167,7 +2202,13 @@ dbus_g_proxy_begin_call_internal (DBusGProxy          *proxy,
                                         timeout))
     goto oom;
   dbus_message_unref (message);
-  g_assert (pending != NULL);
+  
+  /* If we got a NULL pending, that means the connection was disconnected,
+   * and we need to abort this call.  
+   * https://bugs.freedesktop.org/show_bug.cgi?id=12675
+   */
+  if (pending == NULL)
+    return 0;
 
   call_id = ++priv->call_id_counter;
 
@@ -2215,6 +2256,9 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
   n_retvals_processed = 0;
   over = 0;
 
+  /* Keep around a copy of output arguments so we can free on error. */
+  G_VA_COPY(args_unwind, args);
+
   pending = g_hash_table_lookup (priv->pending_calls, GUINT_TO_POINTER (call_id));
   
   dbus_pending_call_block (pending);
@@ -2236,6 +2280,7 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
 	  GValue gvalue = { 0, };
 	  DBusGValueMarshalCtx context;
 
+          context.recursion_depth = 0;
 	  context.gconnection = DBUS_G_CONNECTION_FROM_CONNECTION (priv->manager->connection);
 	  context.proxy = proxy;
 
@@ -2276,7 +2321,7 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
 		goto out;
 
 	      /* Anything that can be demarshaled must be storable */
-	      if (!_dbus_gvalue_store (&gvalue, (gpointer*) return_storage))
+	      if (!_dbus_gvalue_store (&gvalue, return_storage))
 		g_assert_not_reached ();
 	      /* Ownership of the value passes to the client, don't unset */
 	    }
@@ -2319,21 +2364,33 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
 
   ret = TRUE;
  out:
-  va_end (args);
-
   if (ret == FALSE)
     {
       int i;
+
+      valtype = first_arg_type;
       for (i = 0; i < n_retvals_processed; i++)
 	{
+          GValue value = {0,};
 	  gpointer retval;
 
-	  retval = va_arg (args_unwind, gpointer);
+          g_value_init (&value, valtype);
 
-	  g_free (retval);
+	  retval = va_arg (args_unwind, gpointer);
+          if (retval == NULL)
+            {
+              i--;
+              continue;
+            }
+            
+          _dbus_gvalue_take (&value, retval);
+          g_value_unset (&value);
+
+          valtype = va_arg (args_unwind, GType);
 	}
     }
   va_end (args_unwind);
+  va_end (args);
 
   g_hash_table_remove (priv->pending_calls, GUINT_TO_POINTER (call_id));
 
@@ -2376,6 +2433,7 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
   guint call_id;
   va_list args;
   GValueArray *arg_values;
+  DBusGProxyPrivate *priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
   
   g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), NULL);
   g_return_val_if_fail (!DBUS_G_PROXY_DESTROYED (proxy), NULL);
@@ -2384,7 +2442,7 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
 
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (arg_values, first_arg_type, args);
   
-  call_id = dbus_g_proxy_begin_call_internal (proxy, method, notify, user_data, destroy, arg_values,-1);
+  call_id = dbus_g_proxy_begin_call_internal (proxy, method, notify, user_data, destroy, arg_values, priv->default_timeout);
 
   g_value_array_free (arg_values);
 
@@ -2514,20 +2572,33 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
   guint call_id;
   va_list args;
   GValueArray *in_args;
+  DBusGProxyPrivate *priv;
 
   g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), FALSE);
   g_return_val_if_fail (!DBUS_G_PROXY_DESTROYED (proxy), FALSE);
 
+  priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
+
   va_start (args, first_arg_type);
 
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (in_args, first_arg_type, args);
 
-  call_id = dbus_g_proxy_begin_call_internal (proxy, method, NULL, NULL, NULL, in_args,-1);
+  call_id = dbus_g_proxy_begin_call_internal (proxy, method, NULL, NULL, NULL, in_args, priv->default_timeout);
 
   g_value_array_free (in_args);
 
-  first_arg_type = va_arg (args, GType);
-  ret = dbus_g_proxy_end_call_internal (proxy, call_id, error, first_arg_type, args);
+  if (call_id > 0)
+    {
+      first_arg_type = va_arg (args, GType);
+      ret = dbus_g_proxy_end_call_internal (proxy, call_id, error, first_arg_type, args);
+    }
+  else
+    {
+      g_set_error (error, DBUS_GERROR,
+		   DBUS_GERROR_FAILED,
+		   _("Disconnection or out-of-memory"));
+      ret = FALSE;
+    }
 
   va_end (args);
 
@@ -2888,6 +2959,37 @@ dbus_g_proxy_disconnect_signal (DBusGProxy             *proxy,
   g_free (name);
 }
 
+/**
+ * dbus_g_proxy_set_default_timeout:
+ * @proxy: a proxy for a remote interface
+ * @timeout: specify the timeout in milliseconds
+ *
+ * Sets the default timeout to use for a proxy. This timeout will be
+ * used in calls where the timeout is not specified.
+ *
+ * This is useful for long-running operations that takes longer than
+ * the default timeout (which is a on the order of magnitude of tens
+ * of seconds). For some applications, consider using a pattern where
+ * the method returns once the operation is underway
+ * (e.g. immediately) and emits a signal when the operation terminates
+ * (though beware of leaking information with/in the signal return value).
+ *
+ * Since: 0.75
+ */
+void
+dbus_g_proxy_set_default_timeout (DBusGProxy        *proxy,
+                                  int                timeout)
+{
+  DBusGProxyPrivate *priv;
+
+  g_return_if_fail (DBUS_IS_G_PROXY (proxy));
+  g_return_if_fail (!DBUS_G_PROXY_DESTROYED (proxy));
+
+  priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
+  priv->default_timeout = timeout;
+}
+
+
 /** @} End of DBusGLib public */
 
 #ifdef DBUS_BUILD_TESTS
