@@ -34,6 +34,9 @@
 #include <glib/gi18n.h>
 #include "dbus/dbus-signature.h"
 
+/* Seems reasonable, but this should probably be part of the standard protocol */
+#define DBUS_GLIB_MAX_VARIANT_RECURSION 32
+
 static gboolean demarshal_static_variant (DBusGValueMarshalCtx    *context,
 					  DBusMessageIter         *iter,
 					  GValue                  *value,
@@ -82,6 +85,12 @@ static gboolean demarshal_object                (DBusGValueMarshalCtx      *cont
 						 DBusMessageIter           *iter,
 						 GValue                    *value,
 						 GError                   **error);
+static gboolean marshal_signature		(DBusMessageIter           *iter,
+						 const GValue             *value);
+static gboolean demarshal_signature		(DBusGValueMarshalCtx      *context,
+						 DBusMessageIter           *iter,
+						 GValue                    *value,
+						 GError                   **error);
 static gboolean marshal_map                     (DBusMessageIter           *iter,
 						 const GValue              *value);
 static gboolean demarshal_map                   (DBusGValueMarshalCtx      *context,
@@ -333,6 +342,18 @@ _dbus_g_value_types_init (void)
     set_type_metadata (G_TYPE_OBJECT, &typedata);
   }
 
+  {
+    static const DBusGTypeMarshalVtable vtable = {
+      marshal_signature,
+      demarshal_signature
+    };
+    static const DBusGTypeMarshalData typedata = {
+      DBUS_TYPE_SIGNATURE_AS_STRING,
+      &vtable
+    };
+    set_type_metadata (DBUS_TYPE_G_SIGNATURE, &typedata);
+  }
+
   types_initialized = TRUE;
 }
 
@@ -353,6 +374,24 @@ dbus_g_object_path_get_g_type (void)
   return type_id;
 }
 
+/**
+ * Get the GLib type ID for a DBusGSignature boxed type.
+ *
+ * Returns: GLib type
+ */
+GType
+dbus_g_signature_get_g_type (void)
+{
+  static GType type_id = 0;
+
+  if (G_UNLIKELY (type_id == 0))
+    type_id = g_boxed_type_register_static ("DBusGSignature",
+					    (GBoxedCopyFunc) g_strdup,
+					    (GBoxedFreeFunc) g_free);
+
+  return type_id;
+}
+
 
 char *
 _dbus_gtype_to_signature (GType gtype)
@@ -658,7 +697,7 @@ demarshal_object_path (DBusGValueMarshalCtx    *context,
 
   dbus_message_iter_get_basic (iter, &objpath);
 
-  g_value_set_boxed_take_ownership (value, g_strdup (objpath));
+  g_value_set_boxed (value, objpath);
 
   return TRUE;
 }
@@ -701,6 +740,32 @@ demarshal_object (DBusGValueMarshalCtx    *context,
   return TRUE;
 }
 
+static gboolean
+demarshal_signature (DBusGValueMarshalCtx    *context,
+		     DBusMessageIter         *iter,
+		     GValue                  *value,
+		     GError                 **error)
+{
+  const char *sig;
+  int current_type;
+
+  current_type = dbus_message_iter_get_arg_type (iter);
+  if (current_type != DBUS_TYPE_SIGNATURE)
+    {
+      g_set_error (error,
+		   DBUS_GERROR,
+		   DBUS_GERROR_INVALID_ARGS,
+		   _("Expected D-BUS signature, got type code \'%c\'"), (guchar) current_type);
+      return FALSE;
+    }
+
+  dbus_message_iter_get_basic (iter, &sig);
+
+  g_value_set_boxed (value, sig);
+
+  return TRUE;
+}
+
 static gboolean
 demarshal_strv (DBusGValueMarshalCtx    *context,
 		DBusMessageIter         *iter,
@@ -709,9 +774,7 @@ demarshal_strv (DBusGValueMarshalCtx    *context,
 {
   DBusMessageIter subiter;
   int current_type;
-  char **ret;
-  int len;
-  int i;
+  GArray *arr;
 
   current_type = dbus_message_iter_get_arg_type (iter);
   if (current_type != DBUS_TYPE_ARRAY)
@@ -736,24 +799,23 @@ demarshal_strv (DBusGValueMarshalCtx    *context,
       return FALSE;
     }
 
-  len = dbus_message_iter_get_array_len (&subiter);
-  g_assert (len >= 0);
-  ret = g_malloc (sizeof (char *) * (len + 1));
-  
-  i = 0;
+  arr = g_array_new (TRUE, FALSE, sizeof (char *));
+
   while ((current_type = dbus_message_iter_get_arg_type (&subiter)) != DBUS_TYPE_INVALID)
     {
-      g_assert (i < len);
       g_assert (current_type == DBUS_TYPE_STRING);
+      const char *str;
+      char *copy;
       
-      dbus_message_iter_get_basic (&subiter, &(ret[i]));
-      ret[i] = g_strdup (ret[i]);
+      dbus_message_iter_get_basic (&subiter, &str);
+      copy = g_strdup (str);
+      g_array_append_val (arr, copy);
 
       dbus_message_iter_next (&subiter);
-      i++;
     }
-  ret[i] = NULL; 
-  g_value_set_boxed_take_ownership (value, ret);
+
+  g_value_set_boxed_take_ownership (value, arr->data);
+  g_array_free (arr, FALSE);
   
   return TRUE;
 }
@@ -1131,6 +1193,16 @@ _dbus_gvalue_demarshal (DBusGValueMarshalCtx    *context,
 {
   GType gtype;
   DBusGValueDemarshalFunc demarshaller;
+  gboolean retcode = FALSE;
+
+  if (context->recursion_depth > DBUS_GLIB_MAX_VARIANT_RECURSION)
+    {
+      g_set_error (error, DBUS_GERROR,
+                   DBUS_GERROR_NO_MEMORY, 
+                   "Variant recursion limit exceeded");
+      return FALSE;
+    }
+  context->recursion_depth++;
 
   gtype = G_VALUE_TYPE (value);
 
@@ -1143,10 +1215,14 @@ _dbus_gvalue_demarshal (DBusGValueMarshalCtx    *context,
 		   DBUS_GERROR_INVALID_ARGS,
 		   _("No demarshaller registered for type \"%s\""),
 		   g_type_name (gtype));
-      return FALSE;
+    
+      goto out;
     }
   
-  return demarshaller (context, iter, value, error);
+  retcode = demarshaller (context, iter, value, error);
+ out:
+  context->recursion_depth--;  
+  return retcode;
 }
 
 gboolean
@@ -1477,6 +1553,24 @@ marshal_object (DBusMessageIter         *iter,
   return TRUE;
 }
 
+static gboolean
+marshal_signature (DBusMessageIter         *iter,
+		   const GValue            *value)
+{
+  const char *sig;
+
+  g_assert (G_VALUE_TYPE (value) == DBUS_TYPE_G_SIGNATURE);
+
+  sig = (const char*) g_value_get_boxed (value);
+
+  if (!dbus_message_iter_append_basic (iter,
+				       DBUS_TYPE_SIGNATURE,
+				       &sig))
+    return FALSE;
+
+  return TRUE;
+}
+
 struct DBusGLibHashMarshalData
 {
   const char *entry_sig;
