@@ -574,6 +574,13 @@ my_object_echo_variant (MyObject *obj, GValue *variant, GValue *ret, GError **er
     return TRUE;
 }
 
+gboolean
+my_object_echo_signature (MyObject *obj, const gchar *in, gchar **out, GError **error)
+{
+  *out = g_strdup (in);
+  return TRUE;
+}
+
 gboolean 
 my_object_process_variant_of_array_of_ints123 (MyObject *obj, GValue *variant, GError **error)
 {
@@ -647,6 +654,22 @@ my_object_dict_of_dicts (MyObject *obj, GHashTable *in,
   return TRUE;
 }
 
+void
+my_object_dict_of_sigs (MyObject *obj,
+                        GHashTable *dict,
+                        DBusGMethodInvocation *context)
+{
+  dbus_g_method_return (context, dict);
+}
+
+void
+my_object_dict_of_objs (MyObject *obj,
+                        GHashTable *dict,
+                        DBusGMethodInvocation *context)
+{
+  dbus_g_method_return (context, dict);
+}
+
 gboolean
 my_object_emit_frobnicate (MyObject *obj, GError **error)
 {
